@@ -4,6 +4,7 @@
 #ifndef _GI_DISABLE_CAIRO
 #include <cairo-gobject.h>
 #endif
+#include <glib.h>
 #include <glib-object.h>
 #include <gio/gio.h>
 #include <time.h>
@@ -15,6 +16,13 @@
 _GI_TEST_EXTERN
 void regress_set_abort_on_error (gboolean abort_on_error);
 
+/* return annotations */
+_GI_TEST_EXTERN
+char *regress_test_return_allow_none (void);
+
+_GI_TEST_EXTERN
+char *regress_test_return_nullable (void);
+
 /* basic types */
 _GI_TEST_EXTERN
 gboolean regress_test_boolean (gboolean in);
@@ -158,7 +166,7 @@ _GI_TEST_EXTERN
 char *regress_test_array_gtype_in (int n_types, GType *types);
 
 _GI_TEST_EXTERN
-char **regress_test_strv_out_container (void);
+const char **regress_test_strv_out_container (void);
 
 _GI_TEST_EXTERN
 char **regress_test_strv_out (void);
@@ -167,7 +175,7 @@ _GI_TEST_EXTERN
 const char * const * regress_test_strv_out_c (void);
 
 _GI_TEST_EXTERN
-void   regress_test_strv_outarg (char ***retp);
+void   regress_test_strv_outarg (const char ***retp);
 
 _GI_TEST_EXTERN
 int regress_test_array_fixed_size_int_in (int *ints);
@@ -369,7 +377,8 @@ typedef enum
   REGRESS_TEST_VALUE1,
   REGRESS_TEST_VALUE2,
   REGRESS_TEST_VALUE3 = -1,
-  REGRESS_TEST_VALUE4 = '0'
+  REGRESS_TEST_VALUE4 = '0',
+  REGRESS_TEST_VALUE5
 } RegressTestEnum;
 
 typedef enum
@@ -415,6 +424,13 @@ typedef enum
   REGRESS_TEST_EVALUE3 = '0'
 } RegressTestEnumNoGEnum;
 
+/**
+ * REGRESS_TEST_EVALUE_DEPRECATED:
+ *
+ * Scanner used to replace %REGRESS_TEST_EVALUE1 with %REGRESS_TEST_EVALUE_DEPRECATED.
+ */
+#define REGRESS_TEST_EVALUE_DEPRECATED REGRESS_TEST_EVALUE1
+
 _GI_TEST_EXTERN
 const gchar * regress_test_enum_param(RegressTestEnum e);
 
@@ -508,6 +524,8 @@ GQuark regress_atest_error_quark (void);
 #define REGRESS_BOOL_CONSTANT TRUE
 #define REGRESS_G_GINT64_CONSTANT (G_GINT64_CONSTANT (1000))
 #define REGRESS_GUINT64_CONSTANT ((guint64) -1)
+#define REGRESS_GOOD_EXPR_CONSTANT (1 + 2 * (3 + 2) << 3 - 4 >> REGRESS_ATEST_ERROR_CODE2)
+#define REGRESS_BAD_EXPR_CONSTANT (1 + SOMETHING_UNDEFINED)
 
 typedef guint64 RegressTestTypeGUInt64;
 #define REGRESS_GUINT64_CONSTANTA ((RegressTestTypeGUInt64) -1)
@@ -534,6 +552,9 @@ void regress_test_struct_a_clone (RegressTestStructA *a,
 _GI_TEST_EXTERN
 void regress_test_struct_a_parse (RegressTestStructA *a_out, const gchar *string);
 
+_GI_TEST_EXTERN
+void regress_test_array_struct_out (RegressTestStructA **arr, int *len);
+
 struct _RegressTestStructB
 {
   gint8 some_int8;
@@ -597,12 +618,22 @@ struct _RegressTestStructF
   const gint    **const* data4;
   volatile gint  *const data5;
   const gint     *volatile data6;
+  volatile unsigned const char data7;
 };
 
 /* plain-old-data boxed types */
 typedef struct _RegressTestSimpleBoxedA RegressTestSimpleBoxedA;
 typedef struct _RegressTestSimpleBoxedB RegressTestSimpleBoxedB;
 
+/**
+ * RegressTestSimpleBoxedA: (copy-func regress_test_simple_boxed_a_copy)
+ * @some_int:
+ * @some_int8:
+ * @some_double:
+ * @some_enum:
+ *
+ * ...
+ */
 struct _RegressTestSimpleBoxedA
 {
   gint some_int;
@@ -715,6 +746,11 @@ GType regress_test_boxed_c_get_type (void);
 _GI_TEST_EXTERN
 RegressTestBoxedC *regress_test_boxed_c_new (void);
 
+/**
+ * RegressTestBoxedD: (copy-func regress_test_boxed_d_copy)
+ *   (free-func regress_test_boxed_d_free)
+ *
+ */
 typedef struct _RegressTestBoxedD RegressTestBoxedD;
 
 
@@ -761,6 +797,8 @@ struct _RegressTestObj
   double some_double;
   char* string;
   GType gtype;
+  gint name_conflict;
+  GByteArray *byte_array;
 
   /* < private > */
   void (*function_ptr) (void);
@@ -805,6 +843,12 @@ RegressTestObj*   regress_test_obj_new_from_file (const char *x, GError **error)
 _GI_TEST_EXTERN
 void       regress_test_obj_set_bare (RegressTestObj *obj, GObject *bare);
 
+_GI_TEST_EXTERN
+void       regress_test_obj_set_string (RegressTestObj *obj, const char *str);
+
+_GI_TEST_EXTERN
+const char *regress_test_obj_get_string (RegressTestObj *obj);
+
 _GI_TEST_EXTERN
 void       regress_test_obj_emit_sig_with_obj (RegressTestObj *obj);
 
@@ -817,6 +861,18 @@ void       regress_test_obj_emit_sig_with_int64 (RegressTestObj *obj);
 _GI_TEST_EXTERN
 void       regress_test_obj_emit_sig_with_uint64 (RegressTestObj *obj);
 
+_GI_TEST_EXTERN
+void       regress_test_obj_emit_sig_with_array_len_prop (RegressTestObj *obj);
+
+_GI_TEST_EXTERN
+void       regress_test_obj_emit_sig_with_inout_int (RegressTestObj *obj);
+
+_GI_TEST_EXTERN
+void       regress_test_obj_emit_sig_with_error (RegressTestObj *self);
+
+_GI_TEST_EXTERN
+void       regress_test_obj_emit_sig_with_null_error (RegressTestObj *self);
+
 _GI_TEST_EXTERN
 int        regress_test_obj_instance_method (RegressTestObj *obj);
 
@@ -833,6 +889,13 @@ void       regress_forced_method (RegressTestObj *obj);
 _GI_TEST_EXTERN
 void regress_test_array_fixed_out_objects (RegressTestObj ***objs);
 
+#if (defined(__GNUC__) || defined(__clang__)) && defined(__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
+# define G_GCC_STATIC static
+#else
+# define G_GCC_STATIC
+#endif
+_GI_TEST_EXTERN
+void regress_test_array_static_in_int (int x[G_GCC_STATIC 10]);
 
 _GI_TEST_EXTERN
 void regress_test_obj_torture_signature_0 (RegressTestObj    *obj,
@@ -920,6 +983,16 @@ void regress_test_obj_null_out (RegressTestObj **obj);
 
 void regress_func_obj_nullable_in (RegressTestObj *obj);
 
+void regress_test_obj_not_nullable_typed_gpointer_in (RegressTestObj *obj,
+                                                      gpointer        input);
+
+void regress_test_obj_not_nullable_element_typed_gpointer_in (RegressTestObj *obj,
+                                                              gpointer        input,
+                                                              guint           count);
+
+_GI_TEST_EXTERN
+void regress_test_obj_name_conflict (RegressTestObj *obj);
+
 /* inheritance */
 #define REGRESS_TEST_TYPE_SUB_OBJ           (regress_test_sub_obj_get_type ())
 #define REGRESS_TEST_SUB_OBJECT(object)     (G_TYPE_CHECK_INSTANCE_CAST ((object), REGRESS_TEST_TYPE_SUB_OBJ, RegressTestSubObj))
@@ -932,6 +1005,9 @@ typedef struct _RegressTestSubObjClass    RegressTestSubObjClass;
 struct _RegressTestSubObj
 {
   RegressTestObj parent_instance;
+  /*< private >*/
+  gint number;
+  gboolean boolean;
 };
 
 struct _RegressTestSubObjClass
@@ -1019,13 +1095,68 @@ struct _RegressTestFundamentalSubObjectClass {
 _GI_TEST_EXTERN
 GType regress_test_fundamental_sub_object_get_type(void);
 
+struct _RegressTestFundamentalObjectNoGetSetFunc {
+  /* This is not really an RegressTestFundamentalObject but we "inherit"
+   * the structure from it */
+  /* < private > */
+  RegressTestFundamentalObject fundamental_object;
+
+  /* < public > */
+  char *data;
+};
+
+struct _RegressTestFundamentalObjectNoGetSetFuncClass {
+  RegressTestFundamentalObjectClass fundamental_object_class;
+};
+
+typedef struct _RegressTestFundamentalObjectNoGetSetFunc RegressTestFundamentalObjectNoGetSetFunc;
+typedef struct _RegressTestFundamentalObjectNoGetSetFuncClass RegressTestFundamentalObjectNoGetSetFuncClass;
+
+_GI_TEST_EXTERN
+GType regress_test_fundamental_object_no_get_set_func_get_type (void);
+
+struct _RegressTestFundamentalSubObjectNoGetSetFunc {
+  /* < private > */
+  RegressTestFundamentalObjectNoGetSetFunc fundamental_object;
+};
+
+struct _RegressTestFundamentalSubObjectNoGetSetFuncClass {
+  RegressTestFundamentalObjectNoGetSetFuncClass fundamental_object_class;
+};
+
+typedef struct _RegressTestFundamentalSubObjectNoGetSetFunc RegressTestFundamentalSubObjectNoGetSetFunc;
+typedef struct _RegressTestFundamentalSubObjectNoGetSetFuncClass RegressTestFundamentalSubObjectNoGetSetFuncClass;
+
+_GI_TEST_EXTERN
+GType regress_test_fundamental_sub_object_no_get_set_func_get_type (void);
 
 _GI_TEST_EXTERN
 RegressTestFundamentalSubObject *
 regress_test_fundamental_sub_object_new (const char *data);
 
+_GI_TEST_EXTERN
+RegressTestFundamentalObject *
+regress_test_create_fundamental_hidden_class_instance (void);
+
+_GI_TEST_EXTERN
+RegressTestFundamentalObjectNoGetSetFunc *
+regress_test_fundamental_object_no_get_set_func_new (const char *data);
+
+_GI_TEST_EXTERN
+const char *
+regress_test_fundamental_object_no_get_set_func_get_data (RegressTestFundamentalObjectNoGetSetFunc *fundamental);
+
+_GI_TEST_EXTERN
+RegressTestFundamentalSubObjectNoGetSetFunc *
+regress_test_fundamental_sub_object_no_get_set_func_new (const char *data);
+
+_GI_TEST_EXTERN
+void
+regress_test_fundamental_object_no_get_set_func_make_compatible_with_fundamental_sub_object (void);
+
 /* callback */
 typedef void (*RegressTestSimpleCallback) (void);
+typedef void RegressTestNoPtrCallback (void);
 typedef int (*RegressTestCallback) (void);
 typedef int (*RegressTestCallbackUserData) (gpointer user_data);
 /**
@@ -1069,6 +1200,9 @@ typedef void (* RegressTestCallbackArrayInOut) (int **ints, int *length);
 _GI_TEST_EXTERN
 void regress_test_simple_callback (RegressTestSimpleCallback callback);
 
+_GI_TEST_EXTERN
+void regress_test_noptr_callback (RegressTestNoPtrCallback callback);
+
 _GI_TEST_EXTERN
 int regress_test_callback (RegressTestCallback callback);
 
@@ -1156,6 +1290,9 @@ struct _RegressTestInterfaceIface {
 _GI_TEST_EXTERN
 GType regress_test_interface_get_type (void) G_GNUC_CONST;
 
+_GI_TEST_EXTERN
+void regress_test_interface_emit_signal (RegressTestInterface *self);
+
 /* gobject with non-standard prefix */
 #define REGRESS_TEST_TYPE_WI_802_1X              (regress_test_wi_802_1x_get_type ())
 #define REGRESS_TEST_WI_802_1X(object)        (G_TYPE_CHECK_INSTANCE_CAST ((object), REGRESS_TEST_TYPE_WI_802_1X, RegressTestWi8021x))
@@ -1250,6 +1387,9 @@ GValue *regress_test_date_in_gvalue (void);
 _GI_TEST_EXTERN
 GValue *regress_test_strv_in_gvalue (void);
 
+_GI_TEST_EXTERN
+GValue *regress_test_null_strv_in_gvalue (void);
+
 
 _GI_TEST_EXTERN
 GObject * _regress_this_is_a_private_symbol (void);
@@ -1362,7 +1502,7 @@ typedef struct {
 
 _GI_TEST_EXTERN
 void regress_like_xkl_config_item_set_name (RegressLikeXklConfigItem *self,
-                                            const char *name);
+                                            char const *name);
 
 #define REGRESS_UTF8_CONSTANT "const \xe2\x99\xa5 utf8"
 
@@ -1425,4 +1565,45 @@ typedef struct {
   };
 } RegressAnAnonymousUnion;
 
+typedef struct {
+  int x;
+
+  union {
+    struct {
+      RegressLikeGnomeKeyringPasswordSchema *a;
+      RegressLikeXklConfigItem *b;
+    };
+
+    guint padding[4];
+  };
+} RegressAnonymousUnionAndStruct;
+
+GVariant *
+regress_get_variant (void);
+
+typedef struct _RegressTestReferenceCounters RegressTestReferenceCounters;
+
+struct _RegressTestReferenceCounters {
+  grefcount       refcount;
+  gatomicrefcount atomicrefcount;
+};
+
+_GI_TEST_EXTERN
+void regress_test_array_struct_out_none (RegressTestStructA **arr, gsize *len);
+
+_GI_TEST_EXTERN
+void regress_test_array_struct_out_container (RegressTestStructA **arr, gsize *len);
+
+_GI_TEST_EXTERN
+void regress_test_array_struct_out_full_fixed (RegressTestStructA **arr);
+
+_GI_TEST_EXTERN
+void regress_test_array_struct_out_caller_alloc (RegressTestStructA *arr, gsize len);
+
+_GI_TEST_EXTERN
+void regress_test_array_struct_in_full (RegressTestStructA *arr, gsize len);
+
+_GI_TEST_EXTERN
+void regress_test_array_struct_in_none (RegressTestStructA *arr, gsize len);
+
 #endif /* __GITESTTYPES_H__ */
